package SDL::Config;

my $sdl_config; 
$sdl_config = {
                'SFont' => {
                             'SDL_image' => [
                                              '/usr/local/include/SDL',
                                              '/usr/local/lib'
                                            ],
                             'SDL' => [
                                        '/usr/local/include/SDL',
                                        '/usr/local/lib'
                                      ]
                           },
                'OpenGL' => {
                              'GLU' => [
                                         '/usr/X11R6/include/GL',
                                         '/usr/X11R6/lib'
                                       ],
                              'SDL' => [
                                         '/usr/local/include/SDL',
                                         '/usr/local/lib'
                                       ],
                              'GL' => [
                                        '/usr/X11R6/include/GL',
                                        '/usr/X11R6/lib'
                                      ]
                            },
                'SDL' => {
                           'SDL_image' => [
                                            '/usr/local/include/SDL',
                                            '/usr/local/lib'
                                          ],
                           'smpeg' => [
                                        '/usr/local/include/smpeg',
                                        '/usr/local/lib'
                                      ],
                           'SDL_ttf' => [
                                          '/usr/local/include/SDL',
                                          '/usr/local/lib'
                                        ],
                           'SDL' => [
                                      '/usr/local/include/SDL',
                                      '/usr/local/lib'
                                    ],
                           'SDL_mixer' => [
                                            '/usr/local/include/SDL',
                                            '/usr/local/lib'
                                          ],
                           'jpeg' => [
                                       '/usr/local/include',
                                       '/usr/local/lib'
                                     ],
                           'SDL_net' => [
                                          '/usr/local/include/SDL',
                                          '/usr/local/lib'
                                        ],
                           'png' => [
                                      '/usr/local/include',
                                      '/usr/local/lib'
                                    ],
                           'SDL_gfx' => [
                                          '/usr/local/include/SDL',
                                          '/usr/local/lib'
                                        ]
                         }
              };

sub has
{
	my ($class, $define) = @_;
	scalar grep { $$sdl_config{$_}{$define} } keys %$sdl_config;
}

1;
